import unittest
from random import randint
from os import environ
from os import path as os_path
from os.path import getsize
from pathlib import Path
from materials_commons.api import create_project


def fake_name(prefix):
    number = "%05d" % randint(0, 99999)
    return prefix + number


class TestRename(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.project_name = fake_name("TestRenameProject-")
        description = "Test project generated by automated test"
        project = create_project(cls.project_name, description)
        cls.project_id = project.id
        cls.project = project

        cls.top_directory = project.get_top_directory()

        cls.directory_for_rename_name = "/FilesForRename"
        cls.directory_for_rename = project.add_directory(cls.directory_for_rename_name)
        if 'TEST_DATA_DIR' in environ:
            test_path = os_path.abspath(environ['TEST_DATA_DIR'])
            file_path = os_path.join(test_path, 'test_upload_data', 'fractal.jpg')
            path = Path(file_path)
            cls.file_path = str(path.absolute())
            cls.byte_count = getsize(file_path)
            cls.original_filename = "FileForRename.jpg"
            cls.test_file = project.add_file_using_directory(
                cls.directory_for_rename,
                cls.original_filename,
                cls.file_path
            )

    def test_is_setup_correctly(self):
        self.assertTrue('TEST_DATA_DIR' in environ)
        self.assertIsNotNone(self.file_path)
        self.assertTrue(os_path.isfile(self.file_path))
        self.assertIsNotNone(self.test_file)
        self.assertEqual(self.test_file.size, self.byte_count)
        self.assertEqual(self.test_file.name, self.original_filename)

        self.assertIsNotNone(self.project)
        self.assertIsNotNone(self.project.name)
        self.assertEqual(self.project_name, self.project.name)
        self.assertIsNotNone(self.project.id)
        self.assertEqual(self.project_id, self.project.id)
        self.assertEqual(self.top_directory.name, self.project.name)

        directory_list = self.project.get_all_directories()
        self.assertIsNotNone(directory_list)
        print('')
        for directory in directory_list:
            print(directory.name)
        self.assertEqual(len(directory_list), 2)
        self.assertEqual(directory_list[0].name, self.project.name)
        self.assertEqual(directory_list[1].name, self.project.name + self.directory_for_rename_name)

    def test_rename_file(self):
        test_file = self.test_file
        new_name = "NewName.jpg"
        updated_file = test_file.rename(new_name)
        self.assertEqual(updated_file.id, test_file.id)
        self.assertEqual(updated_file.name, new_name)
        self.assertEqual(updated_file._project, self.project)
        directory_list = self.project.get_all_directories()
        directory = directory_list[-1]
        self.assertEqual(directory.id, self.directory_for_rename.id)
        if (directory.shallow):
            directory = self.project.get_directory(directory.id)
        self.assertEqual(directory.id, self.directory_for_rename.id)
        self.assertEqual(directory._project, self.project)
        file_list = directory.get_children()
        probe_file = file_list[0]
        self.assertEqual(probe_file.id, test_file.id)
        self.assertEqual(probe_file.name, new_name)
        self.assertEqual(probe_file._project, self.project)
