import unittest
from random import randint
from materials_commons.api import create_project


def fake_name(prefix):
    number = "%05d" % randint(0, 99999)
    return prefix + number


class TestRename(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.project_name = fake_name("TestRenameProject-")
        description = "Test project generated by automated test"
        project = create_project(cls.project_name, description)
        cls.project_id = project.id
        cls.project = project

        cls.test_base_path = "/TestForRename"
        cls.top_directory = project.get_top_directory()
        cls.test_dir_path_a = "/TestForRename/A"
        cls.directory_a = project.add_directory(cls.test_dir_path_a)
        cls.test_dir_path_b = "/TestForRename/A/B"
        cls.directory_b = project.add_directory(cls.test_dir_path_b)
        cls.test_dir_path_c = "/TestForRename/A/B/C"
        cls.directory_c = project.add_directory(cls.test_dir_path_c)
        cls.test_dir_path_d = "/TestForRename/A/B/D"
        cls.directory_d = project.add_directory(cls.test_dir_path_d)
        cls.test_dir_path_e = "/TestForRename/A/E"
        cls.directory_e = project.add_directory(cls.test_dir_path_e)
        cls.test_dir_path_f = "/TestForRename/F"
        cls.directory_f = project.add_directory(cls.test_dir_path_f)

    def test_is_setup_correctly(self):
        self.assertIsNotNone(self.project)
        self.assertIsNotNone(self.project.name)
        self.assertEqual(self.project_name, self.project.name)
        self.assertIsNotNone(self.project.id)
        self.assertEqual(self.project_id, self.project.id)
        self.assertEqual(self.top_directory.name, self.project.name)
        self.assertEqual(self.directory_a.name, self.project.name + self.test_dir_path_a)
        self.assertEqual(self.directory_b.name, self.project.name + self.test_dir_path_b)

        directory_list = self.project.get_all_directories()
        self.assertIsNotNone(directory_list)
        self.assertEqual(len(directory_list), 8)
        self.assertEqual(directory_list[0].name, self.project.name)
        self.assertEqual(directory_list[1].name, self.project.name + self.test_base_path)
        self.assertEqual(directory_list[2].name, self.project.name + self.test_dir_path_a)
        self.assertEqual(directory_list[3].name, self.project.name + self.test_dir_path_b)
        self.assertEqual(directory_list[4].name, self.project.name + self.test_dir_path_c)
        self.assertEqual(directory_list[5].name, self.project.name + self.test_dir_path_d)
        self.assertEqual(directory_list[6].name, self.project.name + self.test_dir_path_e)
        self.assertEqual(directory_list[7].name, self.project.name + self.test_dir_path_f)

    def test_rename_project(self):
        top_directory = self.project.get_top_directory()
        self.assertEqual(top_directory.path, self.project.name)
        new_name = fake_name("RenamedProject-")
        updated_project = self.project.rename(new_name, description="This project renamed by test code")

        project_name = updated_project.name
        top_directory = updated_project.get_top_directory()
        self.assertEqual(top_directory.path, project_name)
        self.assertEqual(project_name, new_name)

        directory_list = self.project.get_all_directories()
        self.assertIsNotNone(directory_list)
        self.assertEqual(len(directory_list), 8)
        self.assertEqual(directory_list[0].name, project_name)
        self.assertEqual(directory_list[1].name, project_name + self.test_base_path)
        self.assertEqual(directory_list[2].name, project_name + self.test_dir_path_a)
        self.assertEqual(directory_list[3].name, project_name + self.test_dir_path_b)
        self.assertEqual(directory_list[4].name, project_name + self.test_dir_path_c)
        self.assertEqual(directory_list[5].name, project_name + self.test_dir_path_d)
        self.assertEqual(directory_list[6].name, project_name + self.test_dir_path_e)
        self.assertEqual(directory_list[7].name, project_name + self.test_dir_path_f)
